DELIMITER //

DROP FUNCTION IF EXISTS Service_GetStudentAccountID//
CREATE FUNCTION Service_GetStudentAccountID (
    `pStudentID` INT
) RETURNS INT
BEGIN
    RETURN (SELECT students.AccountID
        from students
        where students.id = pStudentID
        limit 1
    );
END//


DROP FUNCTION IF EXISTS Service_SpoofGroups//
CREATE FUNCTION Service_SpoofGroups (
    `pDestRecordBookID` INT,
    `pSrcRecordBookID` INT,
    `pPriority` enum('none','src','dst')
) RETURNS INT
BEGIN
    CASE pPriority

-- no priority
        WHEN 'none' THEN
            UPDATE students_groups
                SET students_groups.RecordBookID = pDestRecordBookID
                WHERE students_groups.RecordBookID = pSrcRecordBookID;

-- destination student priority
        WHEN 'dst' THEN
            UPDATE IGNORE students_groups
                SET students_groups.RecordBookID = pDestRecordBookID
                WHERE students_groups.RecordBookID = pSrcRecordBookID;

-- source student priority
        WHEN 'src' THEN
            INSERT INTO students_groups (RecordBookID, GroupID, SemesterID, Date, State)
                SELECT  pDestRecordBookID,
                        sg.GroupID, sg.SemesterID,
                        sg.Date, sg.State
                FROM students_groups as sg
                WHERE sg.RecordBookID = pSrcRecordBookID
            ON DUPLICATE KEY UPDATE
                students_groups.GroupID = VALUES(GroupID),
                students_groups.Date = VALUES(Date),
                students_groups.State = VALUES(State);
    END CASE;

    DELETE FROM students_groups
        WHERE students_groups.RecordBookID = pSrcRecordBookID;
    RETURN 0;
END //

DROP FUNCTION IF EXISTS Service_SpoofRates//
CREATE FUNCTION Service_SpoofRates (
    `pDestRecordBookID` INT,
    `pSrcRecordBookID` INT,
    `pPriority` enum('none','src','dst')
) RETURNS INT
BEGIN
    CASE pPriority

-- no priority
        WHEN 'none' THEN
            UPDATE rating_table
                SET rating_table.RecordBookID = pDestRecordBookID
                WHERE rating_table.RecordBookID = pSrcRecordBookID;
            UPDATE exam_period_options
                SET exam_period_options.RecordBookID = pDestRecordBookID
                WHERE exam_period_options.RecordBookID = pSrcRecordBookID;

-- destination student priority
        WHEN 'dst' THEN
            UPDATE IGNORE rating_table
                SET rating_table.RecordBookID = pDestRecordBookID
                WHERE rating_table.RecordBookID = pSrcRecordBookID;
            UPDATE IGNORE exam_period_options
                SET exam_period_options.RecordBookID = pDestRecordBookID
                WHERE exam_period_options.RecordBookID = pSrcRecordBookID;

-- source student priority
        WHEN 'src' THEN
            INSERT INTO rating_table (RecordBookID, TeacherID, SubmoduleID, Rate, Date)
                SELECT  pDestRecordBookID,
                        rt.TeacherID, rt.SubmoduleID,
                        rt.Rate, rt.Date
                FROM rating_table as rt
                WHERE rt.RecordBookID = pSrcRecordBookID
            ON DUPLICATE KEY UPDATE
                rating_table.Rate = VALUES(Rate),
                rating_table.Date = VALUES(Date);

            INSERT INTO exam_period_options (SubmoduleID, RecordBookID, Type)
                SELECT  epo.SubmoduleID,
                        pDestRecordBookID,
                        epo.Type
                FROM exam_period_options as epo
                WHERE epo.RecordBookID = pSrcRecordBookID
            ON DUPLICATE KEY UPDATE
                exam_period_options.Type = VALUES(Type);
    END CASE;

    DELETE FROM rating_table
        WHERE rating_table.RecordBookID = pSrcRecordBookID;
    DELETE FROM exam_period_options
        WHERE exam_period_options.RecordBookID = pSrcRecordBookID;
    RETURN 0;
END //

DROP FUNCTION IF EXISTS Service_GlueAccount//
CREATE FUNCTION Service_GlueAccount (
    `pDstAcc` INT,
    `pSrcAcc` INT
    -- determine who dominant in case of duplication
    -- 'none' option on violation just return FALSE
) RETURNS INT
NO SQL
BEGIN
    DECLARE login VARCHAR(50);
    DECLARE mail VARCHAR(64);
    DECLARE pass VARCHAR(255);

    SELECT accounts.Login, accounts.EMail, accounts.Password INTO login, mail, pass
        FROM accounts
        WHERE accounts.id = pSrcAcc LIMIT 1;

    update accounts
    set
        accounts.Login = COALESCE(accounts.Login, login),
        accounts.EMail = COALESCE(accounts.EMail, mail),
        accounts.Password = COALESCE(accounts.Password, pass)
    where accounts.id = pDstAcc;

    RETURN 0;
END //

/**
    GlueStudents(pDestStudentID, pSrcStudentID, pPriority) - объединение дубликатов
    pDestStudentID - ид студента, который останется
    pSrcStudentID - ид студента, данные которого необходимо перенести в pSrcStudentID
    pPriority - приоритет оценок и группы:
        none - выдает ошибку
        src - данные pSrcStudentID приоритетней
        dst - данные pDestStudentID приоритетней
*/


DROP FUNCTION IF EXISTS Service_GlueStudent//
CREATE FUNCTION Service_GlueStudent (
    `pDestStudentID` INT,
    `pSrcStudentID` INT
) RETURNS INT
NO SQL
BEGIN
    DECLARE vt, vDest, vSrc INT DEFAULT 0;
    DECLARE done BOOLEAN DEFAULT FALSE;
	  DECLARE cur CURSOR FOR
	  SELECT rb1.ID as dest, rb2.ID as src
		    FROM (
               select record_books.ID, record_books.ExternalID
                  from record_books
                  where record_books.StudentID = pDestStudentID
             ) as rb1
		    RIGHT JOIN record_books as rb2 ON rb2.ExternalID = rb1.ExternalID
        WHERE rb2.StudentID = pSrcStudentID;
		
	  DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

	  OPEN cur;
	  read_loop: LOOP
		    FETCH cur INTO vDest, vSrc;
		    IF done THEN
		        LEAVE read_loop;
		    END IF;
		    IF vDest IS NULL THEN
			  -- source recordbook has no correspondent recordbook
			  UPDATE record_books
				    SET record_books.StudentID = pDestStudentID
				    WHERE record_books.ID = vSrc;
		    ELSE
			      -- source recordbook has correspondent recordbook
			      SET vt = Service_GlueRecordBooks(vDest, vSrc);
		    END IF;
	  END LOOP;
	  CLOSE cur;
	
    SET vt = Service_GlueAccount(Service_GetStudentAccountID(pDestStudentID), Service_GetStudentAccountID(pSrcStudentID));
    SET vt = Service_DeleteStudent(pSrcStudentID, NULL, TRUE);
    RETURN 0;
END //


DROP FUNCTION IF EXISTS Service_GlueRecordBooks//
CREATE FUNCTION Service_GlueRecordBooks (
    `pDestRecordBookID` INT,
    `pSrcRecordBookID` INT
) RETURNS INT
NO SQL
BEGIN
    DECLARE vt INT DEFAULT 0;
    -- DECLARE EXIT HANDLER FOR SQLEXCEPTION BEGIN
    --     ROLLBACK;
    --     SELECT -1;
    -- END;

    -- START TRANSACTION;
        SET vt = Service_SpoofRates(pDestRecordBookID, pSrcRecordBookID, 'dst');
        SET vt = Service_SpoofGroups(pDestRecordBookID, pSrcRecordBookID, 'dst');

        BEGIN
            DECLARE done INT DEFAULT FALSE;
            DECLARE vDiscID INT;
            DECLARE cur CURSOR FOR
                SELECT disciplines_students.disciplineID
                    FROM disciplines_students
                    WHERE disciplines_students.RecordBookID = pSrcRecordBookID;
            DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

            OPEN cur;
            read_loop: LOOP
                FETCH cur INTO vDiscID;
                IF done THEN
                  LEAVE read_loop;
                END IF;
                SET vt = BindStudent(NULL, vDiscID, pDestRecordBookID);
            END LOOP;
            CLOSE cur;
        END;

        DELETE FROM disciplines_students
            WHERE disciplines_students.RecordBookID = pSrcRecordBookID;

        UPDATE LOW_PRIORITY logs_rating
            SET logs_rating.RecordBookID = pDestRecordBookID
            WHERE logs_rating.RecordBookID = pSrcRecordBookID;
        UPDATE LOW_PRIORITY logs_binds_students
            SET logs_binds_students.RecordBookID = pDestRecordBookID
            WHERE logs_binds_students.RecordBookID = pSrcRecordBookID;
			
		DELETE FROM record_books
			WHERE record_books.Id = pSrcRecordBookID;
    -- COMMIT;

    RETURN 0;
END //


DROP FUNCTION IF EXISTS Service_DeleteStudent //
CREATE FUNCTION Service_DeleteStudent (
    `pStudentID` INT,
	`pRecordBookID` INT,
	`pAccountDeleteMode` TINYINT(1)
) RETURNS INT
NO SQL
BEGIN
    DECLARE vAccountID INT(11) DEFAULT -1;
    CREATE TEMPORARY TABLE vRecBooks AS (
      SELECT record_books.ID
        FROM record_books
        WHERE record_books.StudentID = pStudentID AND record_books.ID = COALESCE(pRecordBookID, record_books.ID));

    DELETE
        FROM disciplines_students
        WHERE disciplines_students.RecordBookID IN (SELECT * FROM vRecBooks);
    DELETE
        FROM rating_table
        WHERE rating_table.RecordBookID IN (SELECT * FROM vRecBooks);
    DELETE
        FROM students_groups
        WHERE students_groups.RecordBookID IN (SELECT * FROM vRecBooks);
    DELETE
        FROM exam_period_options
        WHERE exam_period_options.RecordBookID IN (SELECT * FROM vRecBooks);
    DELETE
        FROM logs_rating
        WHERE logs_rating.RecordBookID IN (SELECT * FROM vRecBooks);
    DELETE
        FROM logs_binds_students
        WHERE logs_binds_students.RecordBookID IN (SELECT * FROM vRecBooks);
	  DELETE
        FROM record_books
        WHERE record_books.StudentID = pStudentID AND record_books.ID = COALESCE(pRecordBookID, record_books.ID);
		
  	IF pRecordBookID IS NOT NULL THEN
		    -- this code executes when caller wishes to delete whole student info
  		  SELECT AccountID INTO vAccountID
	    		  FROM students
			      WHERE students.ID = pStudentID
			  LIMIT 1;

        DELETE
            FROM students
            WHERE students.ID = pStudentID;
        DELETE
            FROM recovery_tokens
            WHERE recovery_tokens.AccountID = vAccountID;
        DELETE
            FROM logs_signin
            WHERE logs_signin.AccountID = vAccountID;

        IF pAccountDeleteMode = TRUE THEN
            DELETE
                FROM accounts
                WHERE accounts.ID = vAccountID;
        END IF;
    END IF;
	
    RETURN 0;
END //




DROP PROCEDURE IF EXISTS Service_MergeAllWeak //
CREATE PROCEDURE Service_MergeAllWeak (
    `pPriority` enum('none','src','dst')
) NO SQL
BEGIN
    DECLARE done INT DEFAULT FALSE;
    DECLARE vRes, stDst, stSrc INT default -1;

    DECLARE cur1 CURSOR FOR
        select  min(students.ID) as recentID,
                max(students.ID) as lastID
        from (	select CONCAT( view_students.LastName, ' ', view_students.FirstName, ' ', view_students.SecondName) as name 
				from view_students
				group by name
				) as t1
        left join accounts on STRCMP(CONCAT( accounts.LastName, ' ', accounts.FirstName, ' ', accounts.SecondName), t1.name) = 0
		join students ON students.AccountID = accounts.ID
        group by CONCAT( accounts.LastName, ' ', accounts.FirstName, ' ', accounts.SecondName)
		having min(students.ID) <> max(students.ID);

    DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;

    OPEN cur1;
    read_loop: LOOP
        FETCH cur1 INTO stDst, stSrc;
        IF done THEN
            LEAVE read_loop;
        END IF;
        BEGIN
            DECLARE CONTINUE HANDLER FOR SQLEXCEPTION ROLLBACK;
            START TRANSACTION;
                SET vRes = Service_GlueStudent(stDst, stSrc, pPriority);
            COMMIT;
        END;
    END LOOP;
    CLOSE cur1;
END //


DELIMITER ;
